searchNodes=[{"doc":"Joken.Hooks implementation for fetching Joken.Signer s from public JWKS URLs. This hook is intended to be used when you are verifying a token is signed with a well known public key. It only overrides the before_verify/2 callback providing a Joken.Signer for the given token. It is important to notice this is not meant for use when GENERATING a token. So, using this hook with Joken.encode_and_sign function WILL NOT WORK!!! To use it, pass this hook to Joken either with the add_hook/2 macro or directly to each Joken function. Example: defmodule MyToken do use Joken.Config add_hook ( JokenJwks , strategy : MyFetchingStrategy ) # rest of your token config end Or: Joken . verify_and_validate ( config , token , nil , context , [ { Joken.Jwks , strategy : MyStrategy } ] ) Fetching strategy Very rarely, your authentication server might rotate or block its keys. Key rotation is the process of issuing a new key that in time will replace the older key. This is security hygiene and should/might be a regular process. Sometimes it is important to block keys because they got leaked or for any other reason. Other times you simply don't control the authentication server and can't ensure the keys won't change. This is the most common scenario for this hook. In these cases (and some others) it is important to have a cache invalidation strategy: all your cached keys should be refreshed. Since the best strategy might differ for each use case, there is a behaviour that can be customized as the &quot;fetching strategy&quot;, that is: when to fetch and re-fetch keys. JokenJwks has a default strategy that tries to be smart and cover most use cases by default. It combines a time based state machine to avoid overflowing the system with re-fetching keys. If that is not a good option for your use case, it can still be configured. Please, see JokenJwks.SignerMatchStrategy or JokenJwks.DefaultStrategyTemplate docs for more information.","ref":"JokenJwks.html","title":"JokenJwks","type":"module"},{"doc":"","ref":"JokenJwks.html#log/3","title":"JokenJwks.log/3","type":"function"},{"doc":"A JokenJwks.SignerMatchStrategy template that has a window of time for refreshing its cache. This is a template and not a concrete implementation. You should use this module in order to use the default strategy. This implementation is a task that should be supervised. It loops on a time window checking whether it should re-fetch keys or not. Every time a bad kid is received it writes to an ets table a counter to 1. When the task loops, it polls for the counter value. If it is more than zero it starts re-fetching the cache. Upon successful fetching, it zeros the counter once again. This way we avoid overloading the JWKS server. It will try to fetch signers when supervision starts it. This can be a sync or async operation depending on the value of first_fetch_sync . It defaults to false . Usage This strategy must be under your apps' supervision tree. It must be explicitly used under a module so that you can have more than one JWKS source. When using this strategy, there is an init_opts/1 callback that can be overridden. This is called upon supervision start. It should return a keyword list with all the options. This follows the standard practice of allowing a callback for using runtime configuration. It can override all other options as this has higher preference. Configuration Other than the init_opts/1 callback you can pass options through Mix.Config and when starting the supervisor. The order of preference in least significant order is: Per environment Mix.Config Supervisor child options init_opts/1 callback The only mandatory option is jwks_url ( binary() ) that is, usually, a runtime parameter like a system environment variable. It is recommended to use the init_opts/1 callback. Other options are: time_interval ( integer() - default 60_000 (1 minute)): time interval for polling if it is needed to re-fetch the keys log_level ( :none | :debug | :info | :warn | :error - default :debug ): the level of log to use for events in the strategy like HTTP errors and so on. It is advised not to turn off logging in production should_start ( boolean() - default true ): whether to start the supervised polling task. For tests, this should be false first_fetch_sync ( boolean() - default false ): whether to fetch the first time synchronously or async explicit_alg ( String.t() ): the JWS algorithm for use with the key. Overrides the one in the JWK http_max_retries_per_fetch ( pos_integer() - default 10 ): passed to Tesla.Middleware.Retry http_delay_per_retry ( pos_integer() - default 500 ): passed to Tesla.Middleware.Retry Examples defmodule JokenExample.MyStrategy do use JokenJwks.DefaultStrategyTemplate def init_opts ( opts ) do url = # fetch url ... Keyword . merge ( opts , jwks_url : url ) end end defmodule JokenExample.Application do @doc false def start ( _type , _args ) do import Supervisor.Spec , warn : false children = [ { MyStrategy , time_interval : 2_000 } ] opts = [ strategy : :one_for_one ] Supervisor . start_link ( children , opts ) end end Then on your token configuration module: defmodule MyToken do use Joken.Config add_hook ( JokenJwks , strategy : MyStrategy ) # rest of your token config end","ref":"JokenJwks.DefaultStrategyTemplate.html","title":"JokenJwks.DefaultStrategyTemplate","type":"module"},{"doc":"Makes a GET request to an OpenID Connect certificates endpoint. This must be a standard JWKS URI as per the specification here: https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata This uses the Tesla library to make it easy to test or change the adapter if wanted. See our tests for an example of mocking the HTTP fetching.","ref":"JokenJwks.HttpFetcher.html","title":"JokenJwks.HttpFetcher","type":"module"},{"doc":"Fetches the JWKS signers from the given url. This retries up to 10 times with a fixed delay of 500 ms until the server delivers an answer. We only perform a GET request that is idempotent. We use :hackney as it validates certificates automatically.","ref":"JokenJwks.HttpFetcher.html#fetch_signers/2","title":"JokenJwks.HttpFetcher.fetch_signers/2","type":"function"},{"doc":"Tesla Middleware for publishing telemetry events. This middleware reports status and request time/response useful for monitoring the JWKS provider.","ref":"JokenJwks.Middleware.Telemetry.html","title":"JokenJwks.Middleware.Telemetry","type":"module"},{"doc":"A strategy behaviour for using with JokenJwks . JokenJwks will call this for every token with a kid. It is the strategy's responsibility to handle caching and matching of the kid with its signers cache. See JokenJwks.DefaultStrategyTemplate for an implementation.","ref":"JokenJwks.SignerMatchStrategy.html","title":"JokenJwks.SignerMatchStrategy","type":"behaviour"},{"doc":"","ref":"JokenJwks.SignerMatchStrategy.html#c:match_signer_for_kid/2","title":"JokenJwks.SignerMatchStrategy.match_signer_for_kid/2","type":"callback"},{"doc":"Changelog All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - [Unreleased]","type":"extras"},{"doc":"Changed Use hackney v1.17.4 Documentation re-organization (#22 thanks to @kianmeng) Conditional telemetry version (#23 thanks to @J3RN)","ref":"changelog.html#1-5-0-2020-12-23","title":"Changelog - [1.5.0] - 2020-12-23","type":"extras"},{"doc":"Changed (Emil Bostijancic) upgrades hackney dependency to 1.16.0 (#17) Updated deps Fixed (@seancribbs) Address unmatched return warnings from Dialyzer (#16)","ref":"changelog.html#1-4-0-2020-09-27","title":"Changelog - [1.4.0] - 2020-09-27","type":"extras"},{"doc":"Fixed (@duzzifelipe) #13 fix: change fetch signers spec","ref":"changelog.html#1-3-1-2020-03-02","title":"Changelog - [1.3.1] - 2020-03-02","type":"extras"},{"doc":"Added (@duzzifelipe) #12 feat: telemetry middleware for tesla","ref":"changelog.html#1-3-0-2020-03-02","title":"Changelog - [1.3.0] - 2020-03-02","type":"extras"},{"doc":"Added (René Mygind Andersen) #7 Adds support for parsing JWKS returned with content-type 'applicat… Changed (@ltj) Use hackney v1.15.2","ref":"changelog.html#1-2-0-2019-11-08","title":"Changelog - [1.2.0] - 2019-11-08","type":"extras"},{"doc":"Added Options for explicitly telling which algorithm will use for the parsed signers; HTTP options for retry and adapter; Integration tests for Google and Microsoft JWKS endpoints. Fixed Fixed docs about how to use DefaultStrategyTemplate and Fixed spelling (#4 thanks to @bforchhammer)","ref":"changelog.html#1-1-0-2019-03-05","title":"Changelog - [1.1.0] - 2019-03-05","type":"extras"},{"doc":"Added First version of the library with a default time window strategy for refetching signers.","ref":"changelog.html#1-0-0-2019-01-02","title":"Changelog - [1.0.0] - 2019-01-02","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work, attach the following boilerplate notice, with the fields enclosed by brackets &quot;[]&quot; replaced with your own identifying information. (Don't include the brackets!) The text should be enclosed in the appropriate comment syntax for the file format. We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third-party archives. Copyright [yyyy] [name of copyright owner] Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html","title":"License","type":"extras"},{"doc":"Joken JWKS A Joken.Hooks implementation that builds a signer out of a JWKS url for verification.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Please see our documentation for usage.","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"def deps do [ { :joken_jwks , &quot;~&gt; 1.5.0&quot; } ] end JWKS (JSON Web Key Set) When using JWTs for digital signing we need a key to sign and verify the contents of the token. Many times the part that is doing signature verification is the same that does the signing and so, it possesses both the private and public key. Other times, this is not the case. When using an authentication server that is different than the &quot;business&quot; server, the later has no control about the keys being used to authenticate a request. On these cases, the authentication server publishes its own public key so that anyone can validate tokens that it has generated before with the matching private key. This scenario is common on delegated authentication as mentioned. One specification that uses this approach is OAuth2 with OpenID Connect. The auth server has a public configuration endpoint where there is plenty information about how to connect with the server. One such information is the JWKS (JSON Web Key Set): a list of public keys the server uses to generate signed tokens. Example of well known JWKS URIs: Google: https://www.googleapis.com/oauth2/v3/certs Microsoft: https://login.microsoftonline.com/common/discovery/keys","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"If your server is using Joken 2 to validate tokens, then you can use this hook to retrieve the list of signers from a JWKS URI. This is how it happens: Build you token configuration (either with use Joken.Config or directly) Initialize your fetching strategy Pass this hook to your verify call: If you are using Joken.Config then it's just a matter of add_hook(JokenJwks, strategy: &lt;&lt;YOUR STRATEGY&gt;&gt;) If you are calling Joken.verify_* directly, you can pass the hook as a last parameter Remember that there can't be a default signer otherwise it will have precedence over this!","ref":"readme.html#joken-jwks-hook","title":"Overview - Joken JWKS hook","type":"extras"},{"doc":"This hook will call the behaviour JokenJwks.SignerMatchstrategy for every token. It is the implementation job to decide how to choose a signer for the token. The only occasion where this will not call the behaviour is if the given token is not properly formed nor it contains a &quot;kid&quot; claim on its header. JWKS mandates this claim so that we know which key to use for verifying. A very naive approach of implementing the callback would be to fetch signers upon startup and then re-fetching every time a token kid does not match with the loaded cache. This could potentially open an attack vector for massively hitting the authentication server. Of course, the auth server JWKS url is public and an attacker could just hit it directly, but it is wise to have some defense mechanism in place when developing your strategy.","ref":"readme.html#architecture","title":"Overview - Architecture","type":"extras"},{"doc":"JokenJwks comes with a smart enough implementation that uses a time window approach for re-fetching signers. By default, it polls the cache state every minute to see if a bad kid was attempted. If so, it re-fetches the cache. So, it will fetch JWKS once every minute tops.","ref":"readme.html#default-strategy-template","title":"Overview - Default Strategy Template","type":"extras"},{"doc":"Since the JWKS specification is just that, a specification, many servers might disagree on how to implement this. For example, Google specifies the &quot;alg&quot; claim on every key instance. Microsoft does not. Therefore we assume some interpretations: Every key must have a &quot;kid&quot; (even if there is only one key) We don't currently check for the &quot;use&quot; claim and so we might hit an encryption key (which will be parsed as well) If no &quot;alg&quot; claim is provided, then the user must pass the option &quot;explicit_alg&quot; That's it for now :)","ref":"readme.html#interpretation-of-the-jwks-rfc","title":"Overview - Interpretation of the JWKS RFC","type":"extras"},{"doc":"Copyright (c) 2018 Victor Nascimento Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#copyright-and-license","title":"Overview - Copyright and License","type":"extras"}]